@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2006-2007 Zeljko Vrba <zvrba@globalnet.hr>
- * Copyright (c) 2006-2007 Alon Bar-Lev <alon.barlev@gmail.com>
+ * Copyright (c) 2006-2011 Alon Bar-Lev <alon.barlev@gmail.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -283,12 +283,10 @@ send_certificate_list (
 		}
 
 		if (
-			(error = common_map_assuan_error (
-				assuan_write_status (
-					ctx,
-					"KEY-FRIEDNLY",
-					nameinfo
-				)
+			(error = assuan_write_status (
+				ctx,
+				"KEY-FRIEDNLY",
+				nameinfo
 			)) != GPG_ERR_NO_ERROR
 		) {
 			goto retry;
@@ -305,12 +303,10 @@ send_certificate_list (
 
 
 			if (
-				(error = common_map_assuan_error (
-					assuan_write_status (
-						ctx,
-						"KEY-FPR",
-						gpginfo
-					)
+				(error = assuan_write_status (
+					ctx,
+					"KEY-FPR",
+					gpginfo
 				)) != GPG_ERR_NO_ERROR
 			) {
 				goto retry;
@@ -320,12 +316,10 @@ send_certificate_list (
 
 		if (
 			!data->config->emulate_openpgp &&
-			(error = common_map_assuan_error (
-				assuan_write_status (
-					ctx,
-					"CERTINFO",
-					info_cert
-				)
+			(error = assuan_write_status (
+				ctx,
+				"CERTINFO",
+				info_cert
 			)) != GPG_ERR_NO_ERROR
 		) {
 			goto retry;
@@ -343,12 +337,10 @@ send_certificate_list (
 			}
 
 			if (
-				(error = common_map_assuan_error (
-					assuan_write_status (
-						ctx,
-						"KEYPAIRINFO",
-						keypairinfo
-					)
+				(error = assuan_write_status (
+					ctx,
+					"KEYPAIRINFO",
+					keypairinfo
 				)) != GPG_ERR_NO_ERROR
 			) {
 				goto retry;
@@ -403,7 +395,7 @@ cleanup:
 
 int _get_certificate_by_type (assuan_context_t ctx, int type, pkcs11h_certificate_id_t *p_cert_id, char **p_key) {
 	cmd_data_t *data = (cmd_data_t *)assuan_get_pointer (ctx);
-	gpg_err_code_t error = GPG_ERR_GENERAL;
+	gpg_err_code_t error = GPG_ERR_BAD_KEY;
 	pkcs11h_certificate_id_list_t user_certificates = NULL;
 	pkcs11h_certificate_id_list_t curr_cert;
 	pkcs11h_certificate_id_t cert_id = NULL;
@@ -481,7 +473,6 @@ int _get_certificate_by_type (assuan_context_t ctx, int type, pkcs11h_certificat
 	}
 
 	if (!found) {
-		error = GPG_ERR_BAD_KEY;
 		goto cleanup;
 	}
 
@@ -526,7 +517,7 @@ void cmd_free_data (assuan_context_t ctx) {
 	}
 }
 
-int cmd_null (assuan_context_t ctx, char *line)
+gpg_error_t cmd_null (assuan_context_t ctx, char *line)
 {
 	(void)ctx;
 	(void)line;
@@ -538,7 +529,7 @@ int cmd_null (assuan_context_t ctx, char *line)
    Returns the card serial number and internally enumerates all certificates.
    This function MUST be called before any other operation with the card.
 */
-int cmd_serialno (assuan_context_t ctx, char *line)
+gpg_error_t cmd_serialno (assuan_context_t ctx, char *line)
 {
 	cmd_data_t *data = (cmd_data_t *)assuan_get_pointer (ctx);
 	gpg_err_code_t error = GPG_ERR_GENERAL;
@@ -587,12 +578,10 @@ int cmd_serialno (assuan_context_t ctx, char *line)
 		}
 
 		if (
-			(error = common_map_assuan_error (
-				assuan_write_status (
-					ctx,
-					"SERIALNO",
-					serial_and_stamp
-				)
+			(error = assuan_write_status (
+				ctx,
+				"SERIALNO",
+				serial_and_stamp
 			)) != GPG_ERR_NO_ERROR
 		) {
 			goto cleanup;
@@ -679,12 +668,10 @@ int cmd_serialno (assuan_context_t ctx, char *line)
 		}
 
 		if (
-			(error = common_map_assuan_error (
-				assuan_write_status (
-					ctx,
-					"SERIALNO",
-					serial_and_stamp
-				)
+			(error = assuan_write_status (
+				ctx,
+				"SERIALNO",
+				serial_and_stamp
 			)) != GPG_ERR_NO_ERROR
 		) {
 			goto retry;
@@ -719,7 +706,7 @@ cleanup:
 }
 
 /** TODO: handle --force option! */
-int cmd_learn (assuan_context_t ctx, char *line)
+gpg_error_t cmd_learn (assuan_context_t ctx, char *line)
 {
 	gpg_err_code_t error = GPG_ERR_GENERAL;
 	pkcs11h_certificate_id_list_t user_certificates = NULL;
@@ -741,26 +728,20 @@ int cmd_learn (assuan_context_t ctx, char *line)
 				&user_certificates
 			)
 		)) != GPG_ERR_NO_ERROR ||
-		(error = common_map_assuan_error (
-			assuan_write_status (
-				ctx,
-				"APPTYPE",
-				"PKCS11"
-			)
+		(error = assuan_write_status (
+			ctx,
+			"APPTYPE",
+			"PKCS11"
 		)) != GPG_ERR_NO_ERROR ||
-		(error = common_map_assuan_error (
-			send_certificate_list (
-				ctx,
-				user_certificates,
-				0
-			)
+		(error = send_certificate_list (
+			ctx,
+			user_certificates,
+			0
 		)) != GPG_ERR_NO_ERROR ||
-		(error = common_map_assuan_error (
-			send_certificate_list (
-				ctx,
-				issuer_certificates,
-				1
-			)
+		(error = send_certificate_list (
+			ctx,
+			issuer_certificates,
+			1
 		)) != GPG_ERR_NO_ERROR
 	) {
 		goto cleanup;
@@ -787,7 +768,7 @@ cleanup:
    Return certificate contents. Line contains the percent-plus escaped
    certificate ID.
 */
-int cmd_readcert (assuan_context_t ctx, char *line)
+gpg_error_t cmd_readcert (assuan_context_t ctx, char *line)
 {
 	gpg_err_code_t error = GPG_ERR_GENERAL;
 	pkcs11h_certificate_id_t cert_id = NULL;
@@ -800,9 +781,7 @@ int cmd_readcert (assuan_context_t ctx, char *line)
 			pkcs11h_certificate_deserializeCertificateId (&cert_id, line)
 		)) != GPG_ERR_NO_ERROR ||
 		(error = get_cert_blob (ctx, cert_id, &blob, &blob_size)) != GPG_ERR_NO_ERROR ||
-		(error = common_map_assuan_error (
-			assuan_send_data (ctx, blob, blob_size)
-		)) != GPG_ERR_NO_ERROR
+		(error = assuan_send_data (ctx, blob, blob_size)) != GPG_ERR_NO_ERROR
 	) {
 		goto cleanup;
 	}
@@ -830,7 +809,7 @@ cleanup:
 }
 
 /** Read key given cert id in line. */
-int cmd_readkey (assuan_context_t ctx, char *line)
+gpg_error_t cmd_readkey (assuan_context_t ctx, char *line)
 {
 	gpg_err_code_t error = GPG_ERR_GENERAL;
 	pkcs11h_certificate_id_t cert_id = NULL;
@@ -863,12 +842,10 @@ int cmd_readkey (assuan_context_t ctx, char *line)
 	}
 
 	if (
-		(error = common_map_assuan_error (
-			assuan_send_data(
-				ctx,
-				blob,
-				gcry_sexp_canon_len (blob, 0, NULL, NULL)
-			)
+		(error = assuan_send_data(
+			ctx,
+			blob,
+			gcry_sexp_canon_len (blob, 0, NULL, NULL)
 		)) != GPG_ERR_NO_ERROR
 	) {
 		goto cleanup;
@@ -897,7 +874,7 @@ cleanup:
 }
 
 /** Store hex-encoded data from line to be signed/decrypted. */
-int cmd_setdata (assuan_context_t ctx, char *line)
+gpg_error_t cmd_setdata (assuan_context_t ctx, char *line)
 {
 	gpg_err_code_t error = GPG_ERR_GENERAL;
 	cmd_data_t *data = (cmd_data_t *)assuan_get_pointer (ctx);
@@ -922,16 +899,30 @@ cleanup:
 }
 
 /** Sign data (set by SETDATA) with certificate id in line. */
-int cmd_pksign (assuan_context_t ctx, char *line)
+gpg_error_t cmd_pksign (assuan_context_t ctx, char *line)
 {
-	static unsigned const char sha1_oid[] = { /* 1.3.14.3.2.26 */
-		0x30, 0x21, 0x30, 0x09, 0x06, 0x05, 0x2b, 0x0e, 0x03,
-		0x02, 0x1a, 0x05, 0x00, 0x04, 0x14
-	};
-	static unsigned const char rmd160_oid[] = { /* is 1.3.36.3.2.1 */
-		0x30, 0x21, 0x30, 0x09, 0x06, 0x05, 0x2b, 0x24, 0x03,
-		0x02, 0x01, 0x05, 0x00, 0x04, 0x14
-	};
+	static unsigned char rmd160_prefix[15] = /* Object ID is 1.3.36.3.2.1 */
+		{ 0x30, 0x21, 0x30, 0x09, 0x06, 0x05, 0x2b, 0x24, 0x03,
+		0x02, 0x01, 0x05, 0x00, 0x04, 0x14  };
+	static unsigned char sha1_prefix[15] =   /* (1.3.14.3.2.26) */
+		{ 0x30, 0x21, 0x30, 0x09, 0x06, 0x05, 0x2b, 0x0e, 0x03,
+		0x02, 0x1a, 0x05, 0x00, 0x04, 0x14  };
+	static unsigned char sha224_prefix[19] = /* (2.16.840.1.101.3.4.2.4) */
+		{ 0x30, 0x2D, 0x30, 0x0d, 0x06, 0x09, 0x60, 0x86, 0x48,
+		0x01, 0x65, 0x03, 0x04, 0x02, 0x04, 0x05, 0x00, 0x04,
+		0x1C  };
+	static unsigned char sha256_prefix[19] = /* (2.16.840.1.101.3.4.2.1) */
+		{ 0x30, 0x31, 0x30, 0x0d, 0x06, 0x09, 0x60, 0x86,
+		0x48, 0x01, 0x65, 0x03, 0x04, 0x02, 0x01, 0x05,
+		0x00, 0x04, 0x20  };
+	static unsigned char sha384_prefix[19] = /* (2.16.840.1.101.3.4.2.2) */
+		{ 0x30, 0x41, 0x30, 0x0d, 0x06, 0x09, 0x60, 0x86,
+		0x48, 0x01, 0x65, 0x03, 0x04, 0x02, 0x02, 0x05,
+		0x00, 0x04, 0x30  };
+	static unsigned char sha512_prefix[19] = /* (2.16.840.1.101.3.4.2.3) */
+		{ 0x30, 0x51, 0x30, 0x0d, 0x06, 0x09, 0x60, 0x86,
+		0x48, 0x01, 0x65, 0x03, 0x04, 0x02, 0x03, 0x05,
+		0x00, 0x04, 0x40  };
 
 	gpg_err_code_t error = GPG_ERR_GENERAL;
 	pkcs11h_certificate_id_t cert_id = NULL;
@@ -942,32 +933,126 @@ int cmd_pksign (assuan_context_t ctx, char *line)
 	int session_locked = 0;
 	unsigned char *sig = NULL;
 	size_t sig_len;
+	char hash[100] = "";
+	enum {
+		INJECT_NONE,
+		INJECT_RMD160,
+		INJECT_SHA1,
+		INJECT_SHA224,
+		INJECT_SHA256,
+		INJECT_SHA384,
+		INJECT_SHA512
+	} inject = INJECT_NONE;
 
 	if (data->data == NULL) {
 		error = GPG_ERR_INV_DATA;
 		goto cleanup;
 	}
 
+	while (*line != '\x0' && (isspace (*line) || *line == '-')) {
+		if (*line == '-') {
+			char *hashprm = "--hash=";
+			char *p = line;
+
+			while (*line != '\x0' && !isspace (*line)) {
+				line++;
+			}
+			line++;
+			
+			if (!strncmp (p, hashprm, strlen (hashprm))) {
+				p += strlen (hashprm);
+				*(line-1) = '\0';
+				snprintf (hash, sizeof(hash), "%s", p);
+			}
+		}
+		else {
+			line++;
+		}
+	}
+
+	if (*line == '\x0') {
+		goto cleanup;
+	}
 	/*
 	 * sender prefixed data with algorithm OID
 	 */
-	if (
-		data->size == 20 + sizeof (sha1_oid) ||
-		data->size == 20 + sizeof (rmd160_oid)
-	) {
-		if (
-			memcmp (data->data, sha1_oid, sizeof (sha1_oid)) &&
-			memcmp (data->data, rmd160_oid, sizeof (rmd160_oid))
-		) {
+	if (strcmp(hash, "")) {
+		if (!strcmp(hash, "rmd160")) {
+			inject = INJECT_RMD160;
+		}
+		else if (!strcmp(hash, "sha1")) {
+			inject = INJECT_SHA1;
+		}
+		else if (!strcmp(hash, "sha224")) {
+			inject = INJECT_SHA224;
+		}
+		else if (!strcmp(hash, "sha256")) {
+			inject = INJECT_SHA256;
+		}
+		else if (!strcmp(hash, "sha384")) {
+			inject = INJECT_SHA384;
+		}
+		else if (!strcmp(hash, "sha512")) {
+			inject = INJECT_SHA512;
+		}
+		else {
 			error = GPG_ERR_UNSUPPORTED_ALGORITHM;
 			goto cleanup;
 		}
 	}
 	else {
-		/*
-		 * unknown hash algorithm;
-		 * gnupg's scdaemon forces to SHA1
-		 */
+		if (
+			data->size == 20 + sizeof (sha1_prefix) ||
+			data->size == 20 + sizeof (rmd160_prefix)
+		) {
+			if (
+				memcmp (data->data, sha1_prefix, sizeof (sha1_prefix)) &&
+				memcmp (data->data, rmd160_prefix, sizeof (rmd160_prefix))
+			) {
+				error = GPG_ERR_UNSUPPORTED_ALGORITHM;
+				goto cleanup;
+			}
+		}
+		else {
+			/*
+			 * unknown hash algorithm;
+			 * gnupg's scdaemon forces to SHA1
+			 */
+			inject = INJECT_SHA1;
+		}
+	}
+
+	if (inject != INJECT_NONE) {
+		const unsigned char *oid;
+		size_t oid_size;
+		switch (inject) {
+			case INJECT_RMD160:
+				oid = rmd160_prefix;
+				oid_size = sizeof (rmd160_prefix);
+			break;
+			case INJECT_SHA1:
+				oid = sha1_prefix;
+				oid_size = sizeof (sha1_prefix);
+			break;
+			case INJECT_SHA224:
+				oid = sha224_prefix;
+				oid_size = sizeof (sha224_prefix);
+			break;
+			case INJECT_SHA256:
+				oid = sha256_prefix;
+				oid_size = sizeof(sha256_prefix);
+			break;
+			case INJECT_SHA384:
+				oid = sha384_prefix;
+				oid_size = sizeof(sha384_prefix);
+			break;
+			case INJECT_SHA512:
+				oid = sha512_prefix;
+				oid_size = sizeof(sha512_prefix);
+			break;
+			default:
+				goto cleanup;
+		}
 
 		need_free__data = 1;
 
@@ -976,14 +1061,14 @@ int cmd_pksign (assuan_context_t ctx, char *line)
 			goto cleanup;
 		}
 
-		if ((_data->data = (unsigned char *)malloc (data->size + sizeof (sha1_oid))) == NULL) {
+		if ((_data->data = (unsigned char *)malloc (data->size + oid_size)) == NULL) {
 			error = GPG_ERR_ENOMEM;
 			goto cleanup;
 		}
 
 		_data->size = 0;
-		memmove (_data->data+_data->size, sha1_oid, sizeof (sha1_oid));
-		_data->size += sizeof (sha1_oid);
+		memmove (_data->data+_data->size, oid, oid_size);
+		_data->size += oid_size;
 		memmove (_data->data+_data->size, data->data, data->size);
 		_data->size += data->size;
 	}
@@ -1064,7 +1149,7 @@ int cmd_pksign (assuan_context_t ctx, char *line)
 				&sig_len
 			)
 		)) != GPG_ERR_NO_ERROR ||
-		(error = common_map_assuan_error (assuan_send_data(ctx, sig, sig_len))) != GPG_ERR_NO_ERROR
+		(error = assuan_send_data(ctx, sig, sig_len)) != GPG_ERR_NO_ERROR
 	) {
 		goto cleanup;
 	}
@@ -1104,7 +1189,7 @@ cleanup:
 }
 
 /** Decrypt data (set by SETDATA) with certificate id in line. */
-int cmd_pkdecrypt (assuan_context_t ctx, char *line)
+gpg_error_t cmd_pkdecrypt (assuan_context_t ctx, char *line)
 {
 	gpg_err_code_t error = GPG_ERR_GENERAL;
 	pkcs11h_certificate_id_t cert_id = NULL;
@@ -1198,9 +1283,7 @@ int cmd_pkdecrypt (assuan_context_t ctx, char *line)
 				&ptext_len
 			)
 		)) != GPG_ERR_NO_ERROR ||
-		(error = common_map_assuan_error (
-			assuan_send_data(ctx, ptext, ptext_len))
-		) != GPG_ERR_NO_ERROR
+		(error = assuan_send_data(ctx, ptext, ptext_len)) != GPG_ERR_NO_ERROR
 	) {
 		goto cleanup;
 	}
@@ -1236,7 +1319,7 @@ cleanup:
    pkcs11-helper neither supports getting random data, nor exports sufficient
    data to use raw PKCS#11.
 */
-int cmd_random (assuan_context_t ctx, char *line)
+gpg_error_t cmd_random (assuan_context_t ctx, char *line)
 {
 	(void)ctx;
 	(void)line;
@@ -1245,7 +1328,7 @@ int cmd_random (assuan_context_t ctx, char *line)
 }
 
 /** Not implemented. */
-int cmd_checkpin (assuan_context_t ctx, char *line)
+gpg_error_t cmd_checkpin (assuan_context_t ctx, char *line)
 {
 	(void)ctx;
 	(void)line;
@@ -1253,18 +1336,18 @@ int cmd_checkpin (assuan_context_t ctx, char *line)
 	return gpg_error (GPG_ERR_INV_OP);
 }
 
-int cmd_getinfo (assuan_context_t ctx, char *line)
+gpg_error_t cmd_getinfo (assuan_context_t ctx, char *line)
 {
 	gpg_err_code_t error = GPG_ERR_GENERAL;
 
 	if (!strcmp (line, "version")) {
 		char *s = PACKAGE_VERSION;
-		error = common_map_assuan_error (assuan_send_data(ctx, s, strlen (s)));
+		error = assuan_send_data(ctx, s, strlen (s));
 	}
 	else if (!strcmp (line, "pid")) {
 		char buf[50];
 		snprintf (buf, sizeof (buf), "%lu", (unsigned long)getpid());
-		error = common_map_assuan_error (assuan_send_data(ctx, buf, strlen (buf)));
+		error = assuan_send_data(ctx, buf, strlen (buf));
 	}
 	else if (!strcmp (line, "socket_name")) {
 		const char *s = scdaemon_get_socket_name ();
@@ -1273,7 +1356,7 @@ int cmd_getinfo (assuan_context_t ctx, char *line)
 			error = GPG_ERR_INV_DATA;
 		}
 		else {
-			error = common_map_assuan_error (assuan_send_data(ctx, s, strlen (s)));
+			error = assuan_send_data(ctx, s, strlen (s));
 		}
 	}
 	else if (!strcmp (line, "status")) {
@@ -1299,7 +1382,7 @@ int cmd_getinfo (assuan_context_t ctx, char *line)
 			}
 		}
 
-		error = common_map_assuan_error (assuan_send_data(ctx, &flag, 1));
+		error = assuan_send_data(ctx, &flag, 1);
 	}
 	else if (!strcmp (line, "reader_list")) {
 		error = GPG_ERR_NO_DATA;
@@ -1311,7 +1394,7 @@ int cmd_getinfo (assuan_context_t ctx, char *line)
 	return gpg_error (error);
 }
 
-int cmd_restart (assuan_context_t ctx, char *line)
+gpg_error_t cmd_restart (assuan_context_t ctx, char *line)
 {
 	(void)ctx;
 	(void)line;
@@ -1319,7 +1402,7 @@ int cmd_restart (assuan_context_t ctx, char *line)
 	return gpg_error (GPG_ERR_NO_ERROR);
 }
 
-int cmd_getattr (assuan_context_t ctx, char *line)
+gpg_error_t cmd_getattr (assuan_context_t ctx, char *line)
 {
 	gpg_err_code_t error = GPG_ERR_GENERAL;
 
@@ -1335,12 +1418,10 @@ int cmd_getattr (assuan_context_t ctx, char *line)
 	}
 	else if (!strcmp (line, "CHV-STATUS")) {
 		if (
-			(error = common_map_assuan_error (
-				assuan_write_status(
-					ctx,
-					"CHV-STATUS",
-					"1 1 1 1 1 1 1"
-				)
+			(error = assuan_write_status(
+				ctx,
+				"CHV-STATUS",
+				"1 1 1 1 1 1 1"
 			)) != GPG_ERR_NO_ERROR
 		) {
 			goto cleanup;
@@ -1348,17 +1429,54 @@ int cmd_getattr (assuan_context_t ctx, char *line)
 	}
 	else if (!strcmp (line, "DISP-NAME")) {
 		if (
-			(error = common_map_assuan_error (
-				assuan_write_status(
-					ctx,
-					"DISP-NAME",
-					"PKCS#11"
-				)
+			(error = assuan_write_status(
+				ctx,
+				"DISP-NAME",
+				"PKCS#11"
 			)) != GPG_ERR_NO_ERROR
 		) {
 			goto cleanup;
 		}
 	}
+	else if (!strcmp (line, "KEY-ATTR")) {
+		int i;
+		for (i=0;i<3;i++) {
+			char buffer[1024];
+
+			/* I am not sure 2048 is right here... */
+			snprintf(buffer, sizeof(buffer), "%d 1 %u %u %d", i+1, GCRY_PK_RSA, 2048, 0);
+
+			if (
+				(error = assuan_write_status(
+					ctx,
+					"KEY-ATTR",
+					buffer
+				)) != GPG_ERR_NO_ERROR
+			) {
+				goto cleanup;
+			}
+		}
+	}
+	else if (!strcmp (line, "EXTCAP")) {
+		int i;
+		for (i=0;i<3;i++) {
+			char buffer[1024];
+
+			/* I am not sure what these are... */
+			snprintf(buffer, sizeof(buffer), "gc=%d ki=%d fc=%d pd=%d mcl3=%u aac=%d sm=%d",
+				0, 0, 0, 0, 2048, 0, 0);
+
+			if (
+				(error = assuan_write_status(
+					ctx,
+					"EXTCAP",
+					buffer
+				)) != GPG_ERR_NO_ERROR
+			) {
+				goto cleanup;
+			}
+		}
+	}
 	else {
 		error = GPG_ERR_INV_DATA;
 		goto cleanup;
@@ -1371,7 +1489,7 @@ cleanup:
 	return gpg_error (error);
 }
 
-int cmd_setattr (assuan_context_t ctx, char *line)
+gpg_error_t cmd_setattr (assuan_context_t ctx, char *line)
 {
 	gpg_err_code_t error = GPG_ERR_GENERAL;
 
@@ -1389,7 +1507,7 @@ cleanup:
 	return gpg_error (error);
 }
 
-int cmd_genkey (assuan_context_t ctx, char *line)
+gpg_error_t cmd_genkey (assuan_context_t ctx, char *line)
 {
 	cmd_data_t *data = (cmd_data_t *)assuan_get_pointer (ctx);
 	gpg_err_code_t error = GPG_ERR_GENERAL;
@@ -1450,24 +1568,20 @@ int cmd_genkey (assuan_context_t ctx, char *line)
 	}
 
 	if (
-		(error = common_map_assuan_error (
-			assuan_write_status (
-				ctx,
-				"KEY-FPR",
-				key
-			)
+		(error = assuan_write_status (
+			ctx,
+			"KEY-FPR",
+			key
 		)) != GPG_ERR_NO_ERROR
 	) {
 		goto cleanup;
 	}
 
 	if (
-		(error = common_map_assuan_error (
-			assuan_write_status(
-				ctx,
-				"KEY-CREATED-AT",
-				timestamp
-			)
+		(error = assuan_write_status(
+			ctx,
+			"KEY-CREATED-AT",
+			timestamp
 		)) != GPG_ERR_NO_ERROR
 	) {
 		goto cleanup;
@@ -1521,24 +1635,20 @@ int cmd_genkey (assuan_context_t ctx, char *line)
 	}
 
 	if (
-		(error = common_map_assuan_error (
-			assuan_write_status(
-				ctx,
-				"KEY-DATA",
-				n_resp
-			)
+		(error = assuan_write_status(
+			ctx,
+			"KEY-DATA",
+			n_resp
 		)) != GPG_ERR_NO_ERROR
 	) {
 		goto cleanup;
 	}
 
 	if (
-		(error = common_map_assuan_error (
-			assuan_write_status(
-				ctx,
-				"KEY-DATA",
-				e_resp
-			)
+		(error = assuan_write_status(
+			ctx,
+			"KEY-DATA",
+			e_resp
 		)) != GPG_ERR_NO_ERROR
 	) {
 		goto cleanup;
