@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2006-2007 Zeljko Vrba <zvrba@globalnet.hr>
- * Copyright (c) 2006-2007 Alon Bar-Lev <alon.barlev@gmail.com>
+ * Copyright (c) 2006-2011 Alon Bar-Lev <alon.barlev@gmail.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -57,6 +57,18 @@
 #include <gnutls/gnutls.h>
 #endif
 
+#ifdef HAVE_W32_SYSTEM
+typedef void *gnupg_fd_t;
+#define GNUPG_INVALID_FD ((void*)(-1))
+#define INT2FD(s) ((void *)(s))
+#define FD2INT(h) ((unsigned int)(h))
+#else
+typedef int gnupg_fd_t;
+#define GNUPG_INVALID_FD (-1)
+#define INT2FD(s) (s)
+#define FD2INT(h) (h)
+#endif
+
 typedef enum {
 	ACCEPT_THREAD_STOP,
 	ACCEPT_THREAD_CLEAN
@@ -95,31 +107,35 @@ register_commands (const assuan_context_t ctx)
 {
 	static struct {
 		const char *name;
-		int (*handler)(assuan_context_t ctx, char *line);
+		assuan_handler_t handler;
+		const char * const help;
 	} table[] = {
-		{ "SERIALNO",	cmd_serialno, },
-		{ "LEARN",	cmd_learn },
-		{ "READCERT",	cmd_readcert },
-		{ "READKEY",	cmd_readkey },
-		{ "SETDATA",	cmd_setdata },
-		{ "PKSIGN",	cmd_pksign },
-		{ "PKAUTH",	NULL },
-		{ "PKDECRYPT",	cmd_pkdecrypt },
-		{ "INPUT",	NULL }, 
-		{ "OUTPUT",	NULL }, 
-		{ "GETATTR",	cmd_getattr },
-		{ "SETATTR",	cmd_setattr },
-		{ "WRITEKEY",	NULL },
-		{ "GENKEY",	cmd_genkey },
-		{ "RANDOM",	NULL },
-		{ "PASSWD",	NULL },
-		{ "CHECKPIN",	cmd_null },
-		{ "LOCK",	NULL },
-		{ "UNLOCK",	NULL },
-		{ "GETINFO",	cmd_getinfo },
-		{ "RESTART",	cmd_restart },
-		{ "CHV-STATUS-1", cmd_null },
-		{ NULL, NULL }
+		{ "SERIALNO",	cmd_serialno, NULL },
+		{ "LEARN",	cmd_learn, NULL },
+		{ "READCERT",	cmd_readcert, NULL },
+		{ "READKEY",	cmd_readkey, NULL },
+		{ "SETDATA",	cmd_setdata, NULL },
+		{ "PKSIGN",	cmd_pksign, NULL },
+		{ "PKAUTH",	NULL, NULL },
+		{ "PKDECRYPT",	cmd_pkdecrypt, NULL },
+		{ "INPUT",	NULL, NULL }, 
+		{ "OUTPUT",	NULL, NULL }, 
+		{ "GETATTR",	cmd_getattr, NULL },
+		{ "SETATTR",	cmd_setattr, NULL },
+		{ "WRITECERT",	NULL, NULL },
+		{ "WRITEKEY",	NULL, NULL },
+		{ "GENKEY",	cmd_genkey, NULL },
+		{ "RANDOM",	NULL, NULL },
+		{ "PASSWD",	NULL, NULL },
+		{ "CHECKPIN",	cmd_null, NULL },
+		{ "LOCK",	NULL, NULL },
+		{ "UNLOCK",	NULL, NULL },
+		{ "GETINFO",	cmd_getinfo, NULL },
+		{ "RESTART",	cmd_restart, NULL },
+		{ "DISCONNECT",	cmd_null, NULL },
+		{ "APDU",	NULL, NULL },
+		{ "CHV-STATUS-1", cmd_null, NULL },	/* gnupg-1.X */
+		{ NULL, NULL, NULL }
 	};
 	int i, ret;
 
@@ -128,7 +144,8 @@ register_commands (const assuan_context_t ctx)
 			(ret = assuan_register_command (
 				ctx,
 				table[i].name,
-				table[i].handler
+				table[i].handler,
+				table[i].help
 			))
 		) {
 			return ret;
@@ -156,24 +173,32 @@ command_handler (const int fd, dconfig_data_t *config)
 	memset (&data, 0, sizeof (data));
 	data.config = config;
 
+	if ((ret = assuan_new(&ctx)) != 0) {
+		common_log (LOG_ERROR,"failed to create assuan context %s", gpg_strerror (ret));
+		goto cleanup;
+	}
+
 	if(fd < 0) {
-		int fds[2] = {0, 1};
-		ret = assuan_init_pipe_server (&ctx, fds);
+		assuan_fd_t fds[2] = {assuan_fdopen(0), assuan_fdopen(1)};
+		ret = assuan_init_pipe_server (ctx, fds);
 	} else {
-		ret = assuan_init_socket_server_ext (&ctx, fd, 2);
+		ret = assuan_init_socket_server (ctx, INT2FD(fd), ASSUAN_SOCKET_SERVER_ACCEPTED);
 	}
 
-	if (ret != ASSUAN_No_Error) {
-		common_log (LOG_FATAL,"failed to initialize server: %s", assuan_strerror (ret));
+	if (ret != 0) {
+		common_log (LOG_ERROR,"failed to initialize server: %s", gpg_strerror (ret));
+		goto cleanup;
 	}
 
-	if(((ret = register_commands(ctx))) != ASSUAN_No_Error) {
-		common_log (LOG_FATAL,"failed to register assuan commands: %s", assuan_strerror (ret));
+	if(((ret = register_commands(ctx))) != 0) {
+		common_log (LOG_ERROR,"failed to register assuan commands: %s", gpg_strerror (ret));
+		goto cleanup;
 	}
 
 	if (config->verbose) {
-		assuan_set_log_stream (ctx, assuan_get_assuan_log_stream ());
+		assuan_set_log_stream (ctx, common_get_log_stream());
 	}
+
 	assuan_set_pointer (ctx, &data);
 
 	while (1) {
@@ -181,20 +206,21 @@ command_handler (const int fd, dconfig_data_t *config)
 			break;
 		}
 
-		if (ret != ASSUAN_No_Error) {
-			common_log (LOG_WARNING,"assuan_accept failed: %s", assuan_strerror(ret));
+		if (ret != 0) {
+			common_log (LOG_WARNING,"assuan_accept failed: %s", gpg_strerror(ret));
 			break;
 		}
 		
-		if ((ret = assuan_process (ctx)) != ASSUAN_No_Error) {
-			common_log (LOG_WARNING,"assuan_process failed: %s", assuan_strerror(ret));
+		if ((ret = assuan_process (ctx)) != 0) {
+			common_log (LOG_WARNING,"assuan_process failed: %s", gpg_strerror(ret));
 		}
 	}
 
-	cmd_free_data (ctx);
+cleanup:
 
 	if (ctx != NULL) {
-		assuan_deinit_server (ctx);
+		cmd_free_data (ctx);
+		assuan_release (ctx);
 		ctx = NULL;
 	}
 }
@@ -216,6 +242,7 @@ server_socket_close (const int fd) {
 		free (s_socket_dir);
 		s_socket_dir = NULL;
 	}
+	assuan_sock_deinit();
 }
 
 static
@@ -245,6 +272,11 @@ server_socket_create (void) {
 	int fd = -1;
 	int rc = -1;
 
+	if ((rc = assuan_sock_init()) != 0) {
+		common_log (LOG_ERROR,"Cannot init socket %s", gpg_strerror (rc));
+		goto cleanup;
+	}
+
 	memset (&serv_addr, 0, sizeof (serv_addr));
 	serv_addr.sun_family = AF_UNIX;
 	assert (strlen (s_socket_name) + 1 < sizeof (serv_addr.sun_path));
@@ -288,7 +320,9 @@ _server_socket_command_handler (void *arg) {
 	command_handler (entry->fd, entry->config);
 	entry->stopped = 1;
 
-	write (s_fd_accept_terminate[1], &clean, sizeof (clean));
+	if (write (s_fd_accept_terminate[1], &clean, sizeof (clean)) == -1) {
+		common_log (LOG_FATAL, "write failed");
+	}
 
 	return NULL;
 }
@@ -428,7 +462,9 @@ static
 void
 server_socket_accept_terminate (pthread_t thread) {
 	accept_command_t stop = ACCEPT_THREAD_STOP;
-	write (s_fd_accept_terminate[1], &stop, sizeof (stop));
+	if (write (s_fd_accept_terminate[1], &stop, sizeof (stop)) == -1) {
+		common_log (LOG_FATAL, "write failed");
+	}
 	pthread_join (thread, NULL);
 	close (s_fd_accept_terminate[0]);
 	close (s_fd_accept_terminate[1]);
@@ -588,7 +624,7 @@ static void usage (const char * const argv0)
 "%s %s\n"
 "\n"
 "Copyright (c) 2006-2007 Zeljko Vrba <zvrba@globalnet.hr>\n"
-"Copyright (c) 2006-2007 Alon Bar-Lev <alon.barlev@gmail.com>\n"
+"Copyright (c) 2006-2011 Alon Bar-Lev <alon.barlev@gmail.com>\n"
 "This program comes with ABSOLUTELY NO WARRANTY.\n"
 "This is free software, and you are welcome to redistribute it\n"
 "under certain conditions. See the file COPYING for details.\n"
@@ -793,7 +829,7 @@ int main (int argc, char *argv[])
 					"%s %s\n"
 					"\n"
 					"Copyright (c) 2006-2007 Zeljko Vrba <zvrba@globalnet.hr>\n"
-					"Copyright (c) 2006-2007 Alon Bar-Lev <alon.barlev@gmail.com>\n"
+					"Copyright (c) 2006-2011 Alon Bar-Lev <alon.barlev@gmail.com>\n"
 					"\n"
 					"This is free software; see the source for copying conditions.\n"
 					"There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n",
@@ -919,11 +955,10 @@ int main (int argc, char *argv[])
 		if (pid != 0) {
 			static const char *key = "SCDAEMON_INFO";
 			char env[1024];
-
-			snprintf (env, sizeof (env), "%s=%s:%lu:1", key, s_socket_name, (unsigned long)pid);
+			snprintf (env, sizeof (env), "%s:%lu:1", s_socket_name, (unsigned long)pid);
 
 			if (argc - base_argc > 0) {
-				putenv (env);
+				setenv(key, env, 1);
 				execvp (argv[base_argc], &(argv[base_argc]));
 				kill (pid, SIGTERM);
 				exit (1);
@@ -931,10 +966,10 @@ int main (int argc, char *argv[])
 			else {
 				if (env_is_csh) {
 					*strchr (env, '=') = ' ';
-					printf ("setenv %s\n", env);
+					printf ("setenv %s %s\n", key, env);
 				}
 				else {
-					printf ("%s; export %s\n", env, key);
+					printf ("%s=%s; export %s\n", key, env, key);
 				}
 				exit (0);
 			}
@@ -954,7 +989,9 @@ int main (int argc, char *argv[])
 			}
 		}
 
-		chdir ("/");
+		if (chdir ("/") == -1) {
+			common_log (LOG_FATAL, "chdir failed");
+		}
 
 		if (argc - base_argc > 0) {
 			struct sigaction sa;
@@ -1037,7 +1074,9 @@ int main (int argc, char *argv[])
 		 */
 		int fds[2];
 		char c;
-		pipe (fds);
+		if (pipe (fds)==-1) {
+			common_log (LOG_FATAL, "Could not create pipe");
+		}
 		close (0);
 		dup2 (fds[0], 0);
 		close (fds[0]);
